/*

 Code for turning on motors and lights in response to light level changes
 read by Phototransistors.
 
 This is free and unencumbered software released into the public domain.
 
 Anyone is free to copy, modify, publish, use, compile, sell, or
 distribute this software, either in source code form or as a compiled
 binary, for any purpose, commercial or non-commercial, and by any
 means.
 
 In jurisdictions that recognize copyright laws, the author or authors
 of this software dedicate any and all copyright interest in the
 software to the public domain. We make this dedication for the benefit
 of the public at large and to the detriment of our heirs and
 successors. We intend this dedication to be an overt act of
 relinquishment in perpetuity of all present and future rights to this
 software under copyright law.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 
 Written by John Keogh of Upsoni, LLC (john.keogh@upsoni.com)
 
 */

int lightOutPin = 12;
int rightForwardOutPin = 4;
int rightReverseOutPin = 5;
int leftForwardOutPin = 6;
int leftReverseOutPin = 7;

int EN1 = 6;  
int EN2 = 5;  
int IN1 = 7;
int IN2 = 4; 

int lightOn = 0;
int rightForwardOn = 0;
int rightReverseOn = 0;
int leftForwardOn = 0;
int leftReverseOn = 0;

int lightPinInitialValue = 0;
int rightForwardInitialValue = 0;
int rightReverseInitialValue = 0;
int leftForwardInitialValue = 0;
int leftReverseInitialValue = 0;

int cutoffValue = 700;

void setup() {
  pinMode(lightOutPin, OUTPUT);
  digitalWrite(lightOutPin, LOW);
  pinMode(rightForwardOutPin, OUTPUT);
  digitalWrite(rightForwardOutPin, LOW);
  pinMode(rightReverseOutPin, OUTPUT);
  digitalWrite(rightReverseOutPin, LOW);
  pinMode(leftForwardOutPin, OUTPUT);
  digitalWrite(leftForwardOutPin, LOW);
  pinMode(leftReverseOutPin, OUTPUT);
  digitalWrite(leftReverseOutPin, LOW);

  pinMode(A0, INPUT);
  pinMode(A1, INPUT);
  pinMode(A2, INPUT);
  pinMode(A3, INPUT);
  pinMode(A4, INPUT);
  pinMode(A5, INPUT);

  digitalWrite(A0, HIGH);
  digitalWrite(A1, HIGH);
  digitalWrite(A2, HIGH);
  digitalWrite(A3, HIGH);
  digitalWrite(A4, HIGH);
  digitalWrite(A5, HIGH);
  
  //use this for debugging, if need be
  Serial.begin(300);
}

// the loop routine runs over and over again forever:
void loop() {
  // read the input on analog pin 0:
  int lightValue = analogRead(A2);
  int leftForwardValue = analogRead(A3);
  int leftReverseValue = analogRead(A4);
  int rightForwardValue = analogRead(A1);
  int rightReverseValue = analogRead(A0);

  //use these for debugging
  //Serial.println("values");
  //Serial.println(lightValue);
  //Serial.println(leftForwardValue);
  //Serial.println(leftReverseValue);
  //Serial.println(rightForwardValue);
  //Serial.println(rightReverseValue);
  //Serial.println("");

  if(lightValue<cutoffValue){
    if(!lightOn){
      Serial.println("light on");
      lightOn=1;
      digitalWrite(lightOutPin, HIGH);   // sets the headlights on
    }
  }
  else{
    if(lightOn){
      Serial.println("light off");
      lightOn=0;
      digitalWrite(lightOutPin, LOW); 
    }
  }

  //the way that this and the code farther down for the control of the 
  //right side works is that it sets either right or left on.  If there
  //is a condition where it appears that both forward and backward should
  //be turned on, it is interpreted as a fault and neither is turned on
  if((leftForwardValue<cutoffValue)&&(leftReverseValue>cutoffValue)){
    if(!leftForwardOn){
      //Serial.println("left forward on");
      leftForwardOn=1;
      leftReverseOn=0;
      Motor1(255, 0);
    }
  }
  else if((leftForwardValue>cutoffValue)&&(leftReverseValue<cutoffValue)){
    if(!leftReverseOn){
      //Serial.println("left reverse on");
      leftForwardOn=0;
      leftReverseOn=1;
      Motor1(255, 1);
    }
  }
  else{
    if((leftForwardOn)||(leftReverseOn)){
      //Serial.println("left going nowhere");
      leftForwardOn=0;
      leftReverseOn=0;
      Motor1(0, 0);
    }
  }

  if((rightForwardValue<cutoffValue)&&(rightReverseValue>cutoffValue)){
    if(!rightForwardOn){
      //Serial.println("right forward on");
      rightForwardOn=1;
      rightReverseOn=0;
      Motor2(255, 0);
    }
  }
  else if((rightForwardValue>cutoffValue)&&(rightReverseValue<cutoffValue)){
    if(!rightReverseOn){
      //Serial.println("right reverse on");
      rightForwardOn=0;
      rightReverseOn=1;
      Motor2(255, 1);
    }
  }
  else{
    if((rightForwardOn)||(rightReverseOn)){
      Serial.println("right going nowhere");
      rightForwardOn=0;
      rightReverseOn=0;
      Motor2(0, 1);
    }
  }

  // print out the value you read:
  delay(1);        // delay in between reads for stability
}

//code for Motor1 and Motor2 is from:
//http://www.dfrobot.com/wiki/index.php/Arduino_Shield
void Motor1(int pwm, boolean reverse)
{
  analogWrite(EN1,pwm); //set pwm control, 0 for stop, and 255 for maximum speed
  if(reverse)
  { 
    digitalWrite(IN1,HIGH);    
  }
  else
  {
    digitalWrite(IN1,LOW);    
  }
}  
         
void Motor2(int pwm, boolean reverse)
{
  analogWrite(EN2,pwm); //set pwm control, 0 for stop, and 255 for maximum speed
  if(reverse)
  { 
    digitalWrite(IN2,HIGH);    
  }
  else
  {
    digitalWrite(IN2,LOW);    
  }
}  

